;;;
;;; $Id: irchat-globals.el,v 1.1 1993/07/12 18:38:06 tmo Exp $
;;;
;;; see file irchat-copyright.el for change log and copyright info

(provide 'irchat-globals)

(defvar irchat-debug-buffer)
(defvar irchat-Command-buffer)
(defvar irchat-Dialogue-buffer)
(defvar irchat-WALLOPS-buffer)
(defvar irchat-IGNORED-buffer)
(defvar irchat-KILLS-buffer)

(defvar irchat-current-channel)
(defvar irchat-current-channels)
(defvar irchat-current-chat-partner)
(defvar irchat-current-chat-partners)
(defvar irchat-channel-indicator)

(defvar irchat-ignore-changes)
(defvar irchat-change-prefix)

(defvar irchat-format-string)
(defvar irchat-format-string1)
(defvar irchat-format-string2)
(defvar irchat-format-string3)
(defvar irchat-format-string4)
(defvar irchat-format-string5)

(defvar irchat-beep-on-bells)
(defvar irchat-server-process)
(defvar irchat-reconnect-automagic)
(defvar irchat-grow-tail)

(defvar irchat-show-wallops)
(defvar irchat-system-fqdname)
(defvar irchat-version)
(defvar irchat-greet-author)
(defvar irchat-author-nickname)

(defvar irchat-freeze)
(defvar irchat-kill-nickname)
(defvar irchat-shorten-kills)
(defvar irchat-receive-convert-list)
(defvar irchat-userathost)
(defvar irchat-ignore-extra-notices)
(defvar irchat-invited-channel)

(defvar irchat-polling)

(defvar irchat-notice-prefix)
(defvar irchat-away-indicator)
(defvar irchat-nick-alist)
(defvar irchat-channel-alist)
(defvar irchat-channel-filter)
(defvar irchat-old-nickname)
;;;
;;;  eof
;;; 